/*!
  @file           IFRConversion_DateConverter.cpp
  @author         D039759
  @ingroup        IFR_DataConv
  @brief          Conversion from/to Date values
  @see            

\if EMIT_LICENCE



    ========== licence begin  GPL
    Copyright (c) 2001-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end






\endif
*/
#include "Interfaces/Runtime/IFR_Common.h"
#include "Interfaces/Runtime/IFR_ErrorCode.h"
#include "Interfaces/Runtime/Conversion/IFRConversion_DateConverter.h"

#define DBUG_CLINK_METHOD_ENTER(x,y) DBUG_CONTEXT_METHOD_ENTER(x, y, &clink)

//----------------------------------------------------------------------
IFRConversion_DateConverter::IFRConversion_DateConverter(IFR_ShortInfo& shortinfo, 
                                                         SAPDBMem_IRawAllocator& allocator)
:IFRConversion_ByteCharDataConverter(shortinfo, allocator, false, false)
{}

//----------------------------------------------------------------------
IFRConversion_DateConverter::~IFRConversion_DateConverter()
{}

//----------------------------------------------------------------------
IFR_Retcode
IFRConversion_DateConverter::translateInput(IFRPacket_DataPart& part,
                                            SQL_TIME_STRUCT& data,
                                            IFR_Length *lengthindicator,
                                            IFR_ConnectionItem& clink)
{
    DBUG_CLINK_METHOD_ENTER(IFRConversion_DateConverter, translateInput);
    clink.error().setRuntimeError(IFR_ERR_CONVERSION_NOT_SUPPORTED);
    DBUG_RETURN(IFR_NOT_OK);
}

//----------------------------------------------------------------------
IFR_Retcode
IFRConversion_DateConverter::translateInput(IFRPacket_DataPart& part,
                                            SQL_TIMESTAMP_STRUCT& data,
                                            IFR_Length *lengthindicator,
                                            IFR_ConnectionItem& clink)
{
    DBUG_CLINK_METHOD_ENTER(IFRConversion_DateConverter, translateInput);
    clink.error().setRuntimeError(IFR_ERR_CONVERSION_NOT_SUPPORTED);
    DBUG_RETURN(IFR_NOT_OK);
}

//----------------------------------------------------------------------
IFR_Retcode
IFRConversion_DateConverter::translateOutput(IFRPacket_DataPart& part,
                                             SQL_TIME_STRUCT& data,
                                             IFR_Length *lengthindicator,
                                             IFR_ConnectionItem& clink)
{
    DBUG_CLINK_METHOD_ENTER(IFRConversion_DateConverter, translateInput);
    clink.error().setRuntimeError(IFR_ERR_CONVERSION_NOT_SUPPORTED);
    DBUG_RETURN(IFR_NOT_OK);
}

//----------------------------------------------------------------------
IFR_Retcode
IFRConversion_DateConverter::translateOutput(IFRPacket_DataPart& part,
                                             SQL_TIMESTAMP_STRUCT& data,
                                             IFR_Length *lengthindicator,
                                             IFR_ConnectionItem& clink)
{
    DBUG_CLINK_METHOD_ENTER(IFRConversion_DateConverter, translateInput);
    clink.error().setRuntimeError(IFR_ERR_CONVERSION_NOT_SUPPORTED);
    DBUG_RETURN(IFR_NOT_OK);
}

#if IFR_NEED_VIRTUAL_DELEGATION
#define TRANSLATE_METHOD_IMPL(xxtype)                                                               \
IFR_Retcode                                                                                         \
IFRConversion_DateConverter::translateInput(IFRPacket_DataPart& part,                               \
                                            xxtype& data,                                           \
                                            IFR_Length *lengthindicator,                            \
                                            IFR_ConnectionItem& clink)                              \
{                                                                                                   \
    return IFRConversion_ByteCharDataConverter::translateInput(part, data, lengthindicator, clink); \
}                                                                                                   \
IFR_Retcode                                                                                         \
IFRConversion_DateConverter::translateOutput(IFRPacket_DataPart& part,                              \
                                            xxtype& data,                                           \
                                            IFR_Length *lengthindicator,                            \
                                            IFR_ConnectionItem& clink)                              \
{                                                                                                   \
    return IFRConversion_ByteCharDataConverter::translateOutput(part, data, lengthindicator, clink);\
}

TRANSLATE_METHOD_IMPL(IFR_Int1)
TRANSLATE_METHOD_IMPL(IFR_Int2)
TRANSLATE_METHOD_IMPL(IFR_Int4)
TRANSLATE_METHOD_IMPL(IFR_Int8)
TRANSLATE_METHOD_IMPL(IFR_UInt1)
TRANSLATE_METHOD_IMPL(IFR_UInt2)
TRANSLATE_METHOD_IMPL(IFR_UInt4)
TRANSLATE_METHOD_IMPL(IFR_UInt8)
TRANSLATE_METHOD_IMPL(double)
TRANSLATE_METHOD_IMPL(float)
TRANSLATE_METHOD_IMPL(SQL_DATE_STRUCT)
TRANSLATE_METHOD_IMPL(SQL_NUMERIC_STRUCT)
TRANSLATE_METHOD_IMPL(GUID)

#endif
